<?php
namespace TYPO3\wp_directmailreturn\Command;

class ReturnAnalysisCommandController extends \TYPO3\CMS\Extbase\Mvc\Controller\CommandController {
    /**
     * Gives an open permission to the deputy after $days
     *
     * @param string $type IMAP, POP3, IMAPS
     * @param int $port port to your Mailserver (default: POP=110, IMAP=143, IMAPS=993) 0 = default port
     * @param string $inbox name of the Inbox
     * @param string $host host of the Mailserver
     * @param string $user Username of the mailbox
     * @param string $password Password of the mailbox
     * @param int $amount Amount of Mails per cycle
     * @param string $mail Email address where error notices should be sent to
     * @return boolean
     */

    public function analyzeCommand($type = 'IMAP', $port = 0, $inbox = 'INBOX', $host = "", $user = "", $password = "", $amount = 300, $email) {
        
        $lockfile = PATH_site.'typo3temp/wp_directmailreturn_cron.lock';
        /** @var $logger \TYPO3\CMS\Core\Log\Logger */
        $logger = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\CMS\Core\Log\LogManager')->getLogger(__CLASS__);

        //Level Definition of the logger
        $levelError = \TYPO3\CMS\Core\Log\LogLevel::ERROR;
        $levelNotice = \TYPO3\CMS\Core\Log\LogLevel::NOTICE;
        $levelInfo = \TYPO3\CMS\Core\Log\LogLevel::INFO;
        $levelWarning = \TYPO3\CMS\Core\Log\LogLevel::WARNING;

        $logger->log($levelInfo, 'TYPO3 wp_directmailreturn Cron: Run started!');

        // Check if cronjob is already running:
        if (@file_exists($lockfile)) {
                        // If the lock is not older than 5 minutes abort execution
                if (filemtime($lockfile) > (time() - (60*5))) {
                        $logger->log($levelError, 'TYPO3 wp_directmailreturn Cron: Aborting, another process is already running!'.chr(10));
                        die('TYPO3 wp_directmailreturn Cron: Aborting, another process is already running!'.chr(10));
                } else {
                       
                        $logger->log($levelNotice, 'TYPO3 wp_directmailreturn Cron: A .lock file was found but it is older than 5 minutes! Processing mails ...'.chr(10));
                        unlink($lockfile);
                }
        }
        touch ($lockfile);

        // create the connection string
        $mb = '';
        switch (strtoupper($type)) {
                case "IMAP":
                        if (trim($port) == 0) $port = '143';
                        $mb = "{".$host.":".$port."}".$inbox;
                        break;
                case "IMAPS":
                        if (trim($port) == 0) $port = '993';
                        $mb = "{".$host.":".$port."/imap/ssl/novalidate-cert"."}".$inbox;
                        break;
                case "POP":
                case "POP3":
                        if (trim($port) == 0) $port = '110';
                        $mb = "{".$host.":".$port."/pop3"."}".$inbox;
                        break;
                default:
                        $mb = $type;
                        $logger->log($levelWarning, 'TYPO3 wp_directmailreturn Cron: WARNING: Unrecognized protocol type: '.$type);
                        break;
        }

        $mbox = imap_open($mb, $user, $password);
        if ($mbox==false) {
                unlink($lockfile);
                $logger->log($levelError, 'TYPO3 wp_directmailreturn Cron: ERROR: Cannot open connection! ('.$mb . " - " . $user . " - " . $password.')');
                mailOnError($email, 'Returnmail failed at '.$_SERVER['HTTP_HOST'],'Dear user, the latest execution of Returnmail ended with an error. Reason: ERROR: Cannot open connection!');
                die('TYPO3 wp_directmailreturn Cron: Cannot open connection! ('.$mb . " - " . $user . " - " . $password.')');
        }
        $mails = imap_sort($mbox, SORTDATE, 0);
        $cnt=0;

        $readMail = $this->objectManager->get('TYPO3\wp_directmailreturn\Utility\ReadMail');

        foreach ($mails as $mail) {
                @set_time_limit(30);
                $content = imap_fetchheader($mbox, $mail).imap_body($mbox, $mail);
                if (trim($content))	{
                        // Split mail into head and content
                        $mailParts = $readMail->extractMailHeader($content);
                        // Find id
                        $midArr = $readMail->find_XTypo3MID($content);
                        if (!is_array($midArr))	{
                                $midArr = $readMail->find_MIDfromReturnPath($mailParts['to']);
                        }
                        // Extract text content
                        $c = trim($readMail->getMessage($mailParts));

                        $cp = $readMail->analyseReturnError($c);
                        $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid,email', 'sys_dmail_maillog', 'rid='.intval($midArr['rid']).' AND rtbl="'.$GLOBALS['TYPO3_DB']->quoteStr($midArr['rtbl'], 'sys_dmail_maillog').'" AND mid='.intval($midArr['mid']).' AND response_type=0');
							  if (!$GLOBALS['TYPO3_DB']->sql_num_rows($res)) {
									
                                $midArr = array();
                                $cp = $mailParts;
                        } else {
                                $row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
                                $midArr['email'] = $row['email'];
                        }

                        $insertFields = array(
                                'tstamp' => time(),
                                'response_type' => -127,
                                'mid' => intval($midArr['mid']),
                                'rid' => intval($midArr['rid']),
                                'email' => $midArr['email'],
                                'rtbl' => $midArr['rtbl'],
                                'return_content' => serialize($cp),
                                'return_code' => intval($cp['reason'])
                        );
                        $GLOBALS['TYPO3_DB']->exec_INSERTquery('sys_dmail_maillog', $insertFields);
                }

                imap_delete($mbox, $mail);
                $cnt++;
                if ($cnt >= $amount) break;
        }
        imap_close($mbox, CL_EXPUNGE);
        unlink($lockfile);
        $logger->log($levelNotice, 'TYPO3 wp_directmailreturn Cron: Run ended successfully! '.$cnt.' mails have been analysed!');
        return true;
    }

     /*
     * send a mail with build-in swiftmailer
     * @param \mixed $to array(key1 => array('email' => 'name1@domain.tld', 'name' => 'Name1'), key2 => array('email' => 'name2@domain.tld', 'name' => 'Name2')) or just a string with the email-address
     * @param \string $subject
     * @param \string $plain
     * @return \boolean true, if mail should be send - false, if parameter errors are given
     */
    public function mailOnError($to, $subject, $plain) {
       if(strcmp($to, '')!=0)
       {
            $fromEmail = 'noreply@'.$_SERVER['HTTP_HOST'];
            $fromName = $_SERVER['HTTP_HOST'];
            // make instance of swiftmailer
          $message = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Mail\\MailMessage');

          // from
          if ($fromEmail) {
            $message->setFrom(array($fromEmail => $fromName));
          }

          // to
          $recipients = array();
          if (is_array($to)) {
            foreach ($to as $pair) {
              if (\TYPO3\CMS\Core\Utility\GeneralUtility::validEmail($pair['email'], false)) {
                if (trim($pair['name'])) {
                  $recipients[$pair['email']] = $pair['name'];
                } else {
                  $recipients[] = $pair['email'];
                }
              }
            }
          } else {
            $recipients[] = $to;
          }

          if (!count($recipients)) {
            return false;
          }

          $message->setTo($recipients);

          // subject
          $message->setSubject($subject);

          // html
          $message->setBody($html, 'text/html', 'utf-8');

          // plain
          if ($plain) {
            $message->addPart($plain, 'text/plain', 'utf-8');
          }

          // send
          $message->send();
       }

          return true;
    }
}
?>
